@@ -37,7 +37,6 @@ Learn more about `caching algorithms on Wikipedia
 import heapq
 import weakref
 import itertools
-from collections import deque
 from operator import attrgetter
 
 try:
@@ -70,17 +69,16 @@ PREV, NEXT, KEY, VALUE = range(4)   # names for the link fields
 DEFAULT_MAX_SIZE = 128
 
 
-class LRU(dict):
-    """The ``LRU`` is :class:`dict` subtype implementation of the
-    *Least-Recently Used* caching strategy.
+class LRI(dict):
+    """The ``LRI`` implements the basic *Least Recently Inserted* strategy to
+    caching. One could also think of this as a ``SizeLimitedDefaultDict``.
 
-    Args:
-        max_size (int): Max number of items to cache. Defaults to ``128``.
-        values (iterable): Initial values for the cache. Defaults to ``None``.
-        on_miss (callable): a callable which accepts a single argument, the
-            key not present in the cache, and returns the value to be cached.
+    *on_miss* is a callable that accepts the missing key (as opposed
+    to :class:`collections.defaultdict`'s "default_factory", which
+    accepts no arguments.) Also note that, like the :class:`LRI`,
+    the ``LRI`` is instrumented with statistics tracking.
 
-    >>> cap_cache = LRU(max_size=2)
+    >>> cap_cache = LRI(max_size=2)
     >>> cap_cache['a'], cap_cache['b'] = 'A', 'B'
     >>> from pprint import pprint as pp
     >>> pp(dict(cap_cache))
@@ -90,19 +88,8 @@ class LRU(dict):
     >>> cap_cache['c'] = 'C'
     >>> print(cap_cache.get('a'))
     None
-
-    This cache is also instrumented with statistics
-    collection. ``hit_count``, ``miss_count``, and ``soft_miss_count``
-    are all integer members that can be used to introspect the
-    performance of the cache. ("Soft" misses are misses that did not
-    raise :exc:`KeyError`, e.g., ``LRU.get()`` or ``on_miss`` was used to
-    cache a default.
-
     >>> cap_cache.hit_count, cap_cache.miss_count, cap_cache.soft_miss_count
     (3, 1, 1)
-
-    Other than the size-limiting caching behavior and statistics,
-    ``LRU`` acts like its parent class, the built-in Python :class:`dict`.
     """
     def __init__(self, max_size=DEFAULT_MAX_SIZE, values=None,
                  on_miss=None):
@@ -139,15 +126,21 @@ class LRU(dict):
         self._anchor = anchor
 
     def _print_ll(self):
-        link = self._anchor
         print('***')
+        for (key, val) in self._get_flattened_ll():
+            print(key, val)
+        print('***')
+        return
+
+    def _get_flattened_ll(self):
+        flattened_list = []
+        link = self._anchor
         while True:
-            print(link[KEY], link[VALUE])
+            flattened_list.append((link[KEY], link[VALUE]))
             link = link[NEXT]
             if link is self._anchor:
                 break
-        print('***')
-        return
+        return flattened_list
 
     def _get_link_and_move_to_front_of_ll(self, key):
         # find what will become the newest link. this may raise a
@@ -210,15 +203,15 @@ class LRU(dict):
                     self._set_key_and_add_to_front_of_ll(key, value)
                 else:
                     evicted = self._set_key_and_evict_last_in_ll(key, value)
-                    super(LRU, self).__delitem__(evicted)
-                super(LRU, self).__setitem__(key, value)
+                    super(LRI, self).__delitem__(evicted)
+                super(LRI, self).__setitem__(key, value)
             else:
                 link[VALUE] = value
 
     def __getitem__(self, key):
         with self._lock:
             try:
-                link = self._get_link_and_move_to_front_of_ll(key)
+                link = self._link_lookup[key]
             except KeyError:
                 self.miss_count += 1
                 if not self.on_miss:
@@ -238,14 +231,14 @@ class LRU(dict):
 
     def __delitem__(self, key):
         with self._lock:
-            super(LRU, self).__delitem__(key)
+            super(LRI, self).__delitem__(key)
             self._remove_from_ll(key)
 
     def pop(self, key, default=_MISSING):
         # NB: hit/miss counts are bypassed for pop()
         with self._lock:
             try:
-                ret = super(LRU, self).pop(key)
+                ret = super(LRI, self).pop(key)
             except KeyError:
                 if default is _MISSING:
                     raise
@@ -256,13 +249,13 @@ class LRU(dict):
 
     def popitem(self):
         with self._lock:
-            item = super(LRU, self).popitem()
+            item = super(LRI, self).popitem()
             self._remove_from_ll(item[0])
             return item
 
     def clear(self):
         with self._lock:
-            super(LRU, self).clear()
+            super(LRI, self).clear()
             self._init_ll()
 
     def copy(self):
@@ -299,112 +292,67 @@ class LRU(dict):
                 return True
             if len(other) != len(self):
                 return False
-            if not isinstance(other, LRU):
+            if not isinstance(other, LRI):
                 return other == self
-            return super(LRU, self).__eq__(other)
+            return super(LRI, self).__eq__(other)
 
     def __ne__(self, other):
         return not (self == other)
 
     def __repr__(self):
         cn = self.__class__.__name__
-        val_map = super(LRU, self).__repr__()
+        val_map = super(LRI, self).__repr__()
         return ('%s(max_size=%r, on_miss=%r, values=%s)'
                 % (cn, self.max_size, self.on_miss, val_map))
 
 
-class LRI(dict):
-    """The ``LRI`` implements the basic *Least Recently Inserted* strategy to
-    caching. One could also think of this as a ``SizeLimitedDefaultDict``.
+class LRU(LRI):
+    """The ``LRU`` is :class:`dict` subtype implementation of the
+    *Least-Recently Used* caching strategy.
 
-    *on_miss* is a callable that accepts the missing key (as opposed
-    to :class:`collections.defaultdict`'s "default_factory", which
-    accepts no arguments.) Also note that, like the :class:`LRU`,
-    the ``LRI`` is instrumented with statistics tracking.
+    Args:
+        max_size (int): Max number of items to cache. Defaults to ``128``.
+        values (iterable): Initial values for the cache. Defaults to ``None``.
+        on_miss (callable): a callable which accepts a single argument, the
+            key not present in the cache, and returns the value to be cached.
 
-    >>> cap_cache = LRI(max_size=2)
+    >>> cap_cache = LRU(max_size=2)
     >>> cap_cache['a'], cap_cache['b'] = 'A', 'B'
     >>> from pprint import pprint as pp
-    >>> pp(cap_cache)
+    >>> pp(dict(cap_cache))
     {'a': 'A', 'b': 'B'}
     >>> [cap_cache['b'] for i in range(3)][0]
     'B'
     >>> cap_cache['c'] = 'C'
     >>> print(cap_cache.get('a'))
     None
+
+    This cache is also instrumented with statistics
+    collection. ``hit_count``, ``miss_count``, and ``soft_miss_count``
+    are all integer members that can be used to introspect the
+    performance of the cache. ("Soft" misses are misses that did not
+    raise :exc:`KeyError`, e.g., ``LRU.get()`` or ``on_miss`` was used to
+    cache a default.
+
     >>> cap_cache.hit_count, cap_cache.miss_count, cap_cache.soft_miss_count
     (3, 1, 1)
+
+    Other than the size-limiting caching behavior and statistics,
+    ``LRU`` acts like its parent class, the built-in Python :class:`dict`.
     """
-    # In order to support delitem andn .pop() setitem will need to
-    # popleft until it finds a key still in the cache. or, only
-    # support popitems and raise an error on pop.
-    def __init__(self, max_size=DEFAULT_MAX_SIZE, values=None,
-                 on_miss=None):
-        super(LRI, self).__init__()
-        self.hit_count = self.miss_count = self.soft_miss_count = 0
-        self.max_size = max_size
-        self.on_miss = on_miss
-        self._queue = deque()
-
-        if values:
-            self.update(values)
-
-    def __setitem__(self, key, value):
-        # TODO: pop support (see above)
-        if len(self) >= self.max_size:
-            old = self._queue.popleft()
-            del self[old]
-        super(LRI, self).__setitem__(key, value)
-        self._queue.append(key)
-
-    def update(self, E, **F):
-        # E and F are throwback names to the dict() __doc__
-        if E is self:
-            return
-        setitem = self.__setitem__
-        if callable(getattr(E, 'keys', None)):
-            for k in E.keys():
-                setitem(k, E[k])
-        else:
-            for k, v in E:
-                setitem(k, v)
-        for k in F:
-            setitem(k, F[k])
-        return
-
-    def copy(self):
-        return self.__class__(max_size=self.max_size, values=self)
-
-    def clear(self):
-        self._queue.clear()
-        super(LRI, self).clear()
-
     def __getitem__(self, key):
-        try:
-            ret = super(LRI, self).__getitem__(key)
-        except KeyError:
-            self.miss_count += 1
-            if not self.on_miss:
-                raise
-            ret = self[key] = self.on_miss(key)
-            return ret
-        self.hit_count += 1
-        return ret
+        with self._lock:
+            try:
+                link = self._get_link_and_move_to_front_of_ll(key)
+            except KeyError:
+                self.miss_count += 1
+                if not self.on_miss:
+                    raise
+                ret = self[key] = self.on_miss(key)
+                return ret
 
-    def get(self, key, default=None):
-        try:
-            return self[key]
-        except KeyError:
-            self.soft_miss_count += 1
-            return default
-
-    def setdefault(self, key, default=None):
-        try:
-            return self[key]
-        except KeyError:
-            self.soft_miss_count += 1
-            self[key] = default
-            return default
+            self.hit_count += 1
+            return link[VALUE]
 
 
 ### Cached decorator
@@ -511,6 +459,7 @@ class CachedMethod(object):
     """
     def __init__(self, func, cache, scoped=True, typed=False, key=None):
         self.func = func
+        self.__isabstractmethod__ = getattr(func, '__isabstractmethod__', False)
         if isinstance(cache, basestring):
             self.get_cache = attrgetter(cache)
         elif callable(cache):
@@ -652,6 +601,7 @@ class cachedproperty(object):
     """
     def __init__(self, func):
         self.__doc__ = getattr(func, '__doc__')
+        self.__isabstractmethod__ = getattr(func, '__isabstractmethod__', False)
         self.func = func
 
     def __get__(self, obj, objtype=None):
