@@ -27,15 +27,15 @@ import sys
 import linecache
 
 
-if str is bytes:  # py2
-    text = unicode
-else:  # py3
-    text = str
+try:
+    text = unicode  # Python 2
+except NameError:
+    text = str      # Python 3
 
 
 # TODO: chaining primitives?  what are real use cases where these help?
 
-# TODO: print_* for backwards compatability
+# TODO: print_* for backwards compatibility
 # __all__ = ['extract_stack', 'extract_tb', 'format_exception',
 #            'format_exception_only', 'format_list', 'format_stack',
 #            'format_tb', 'print_exc', 'format_exc', 'print_exception',
@@ -601,7 +601,7 @@ def _some_str(value):
     except Exception:
         pass
     try:
-        value = unicode(value)
+        value = text(value)
         return value.encode("ascii", "backslashreplace")
     except Exception:
         pass
