@@ -26,15 +26,15 @@ def test_wraps_basic():
 
     @pita_wrap(flag=True)
     def simple_func():
-        "my doc string"
+        '''"""a tricky docstring"""'''
         return 'hello'
 
     assert simple_func() == (True, 'simple_func', 'hello')
-    assert simple_func.__doc__ == "my doc string"
+    assert simple_func.__doc__ == '''"""a tricky docstring"""'''
 
     assert callable(simple_func.__wrapped__)
     assert simple_func.__wrapped__() == 'hello'
-    assert simple_func.__wrapped__.__doc__ == "my doc string"
+    assert simple_func.__wrapped__.__doc__ == '''"""a tricky docstring"""'''
 
     @pita_wrap(flag=False)
     def less_simple_func(arg='hello'):
@@ -175,3 +175,99 @@ def test_wraps_wrappers():
                 return cls(x + y)
 
     return
+
+
+def test_FunctionBuilder_add_arg():
+    fb = FunctionBuilder('return_five', doc='returns the integer 5',
+                         body='return 5')
+    f = fb.get_func()
+    assert f() == 5
+
+    fb.add_arg('val')
+    f = fb.get_func()
+    assert f(val='ignored') == 5
+
+    with pytest.raises(ValueError) as excinfo:
+        fb.add_arg('val')
+    excinfo.typename == 'ExistingArgument'
+
+    fb = FunctionBuilder('return_val', doc='returns the value',
+                         body='return val')
+
+    broken_func = fb.get_func()
+    with pytest.raises(NameError):
+        broken_func()
+
+    fb.add_arg('val', default='default_val')
+
+    better_func = fb.get_func()
+    assert better_func() == 'default_val'
+
+    assert better_func('positional') == 'positional'
+    assert better_func(val='keyword') == 'keyword'
+
+
+def test_wraps_expected():
+    def expect_string(func):
+        @wraps(func, expected="c")
+        def wrapped(*args, **kwargs):
+            args, c = args[:2], args[-1]
+            return func(*args, **kwargs) + (c,)
+        return wrapped
+
+    expected_string = expect_string(wrappable_func)
+    assert expected_string(1, 2, 3) == (1, 2, 3)
+
+    with pytest.raises(TypeError) as excinfo:
+        expected_string(1, 2)
+
+    # a rough way of making sure we got the kind of error we expected
+    assert 'argument' in repr(excinfo.value)
+
+    def expect_list(func):
+        @wraps(func, expected=["c"])
+        def wrapped(*args, **kwargs):
+            args, c = args[:2], args[-1]
+            return func(*args, **kwargs) + (c,)
+        return wrapped
+
+    assert expect_list(wrappable_func)(1, 2, c=4) == (1, 2, 4)
+
+    def expect_pair(func):
+        @wraps(func, expected=[('c', 5)])
+        def wrapped(*args, **kwargs):
+            args, c = args[:2], args[-1]
+            return func(*args, **kwargs) + (c,)
+        return wrapped
+
+    assert expect_pair(wrappable_func)(1, 2) == (1, 2, 5)
+
+    def expect_dict(func):
+        @wraps(func, expected={'c': 6})
+        def wrapped(*args, **kwargs):
+            args, c = args[:2], args[-1]
+            return func(*args, **kwargs) + (c,)
+        return wrapped
+
+    assert expect_dict(wrappable_func)(1, 2) == (1, 2, 6)
+
+
+def test_defaults_dict():
+    def example(req, test='default'):
+        return req
+
+    fb_example = FunctionBuilder.from_func(example)
+    assert 'test' in fb_example.args
+    dd = fb_example.get_defaults_dict()
+    assert dd['test'] == 'default'
+    assert 'req' not in dd
+
+
+def test_get_arg_names():
+    def example(req, test='default'):
+        return req
+
+    fb_example = FunctionBuilder.from_func(example)
+    assert 'test' in fb_example.args
+    assert fb_example.get_arg_names() == ('req', 'test')
+    assert fb_example.get_arg_names(only_required=True) == ('req',)
